package com.github.jfcloud.gene.sample.service.biz;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.file.entity.FileInfo;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.file.vo.FileVo;
import com.github.jfcloud.gene.flow.service.FlowAuditService;
import com.github.jfcloud.gene.sample.entity.PathologicalPreExperiment;
import com.github.jfcloud.gene.sample.entity.SampleInfo;
import com.github.jfcloud.gene.sample.entity.SamplePathological;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.mapper.SamplePathologicalMapper;
import com.github.jfcloud.gene.sample.service.PathologicalPreExperimentService;
import com.github.jfcloud.gene.sample.service.SampleEditService;
import com.github.jfcloud.gene.sample.service.SamplePathologicalService;
import com.github.jfcloud.gene.sample.vo.PreExperimentVo;
import com.github.jfcloud.gene.sample.vo.SamplePathologicalVo;
import com.github.jfcloud.gene.sample.vo.SampleSubmitVo;
import com.github.jfcloud.gene.util.WordDataService;
import com.github.jfcloud.rich.vo.HtmlParseVo;
import com.github.jfcloud.rich.word.WordNodeFactory;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class SamplePathologicalServiceImpl extends ServiceImpl<SamplePathologicalMapper, SamplePathological> implements SamplePathologicalService, SampleEditService {

    /**
     * 病理样本-样本信息附件类型
     */
    public static final String PATHOLOGICAL_SAMPLE_INFO = "pathological.sampleInfo";
    /**
     * 病理样本-抗体信息附件类型
     */
    public static final String PATHOLOGICAL_ANTIBODY_INFO = "pathological.antibodyInfo";

    private final SampleInfoMapper sampleInfoMapper;
    private final PathologicalPreExperimentService preExperimentService;
    private final FileInfoService fileInfoService;
    private final WordDataService wordDataService;
    private final UserIdNameCache userIdNameCache;
    private final FlowAuditService flowAuditService;

    @Override
    public void removeBySampleId(Long sampleId) {
        List<SamplePathological> pathologicalList = list(new LambdaQueryWrapper<>(SamplePathological.class)
                .select(SamplePathological::getId)
                .eq(SamplePathological::getSampleId, sampleId));
        if (pathologicalList.isEmpty()) {
            return;
        }

        Set<Long> idSet = pathologicalList.stream().map(SamplePathological::getId).collect(Collectors.toSet());
        removeBatchByIds(idSet);

        preExperimentService.remove(new LambdaQueryWrapper<>(PathologicalPreExperiment.class).in(PathologicalPreExperiment::getPathologicalId, idSet));
        fileInfoService.remove(new LambdaQueryWrapper<>(FileInfo.class).in(FileInfo::getRelateId, idSet));
    }

    @Override
    public void save(Long sampleId, SampleSubmitVo vo) {
        SamplePathologicalVo pathological = vo.getPathological();
        Assert.notNull(pathological, "病理样本送检不能为空");
        removeBySampleId(sampleId);

        //保存病理样本送检信息
        SamplePathological samplePathological = BeanUtil.copyProperties(pathological, SamplePathological.class);
        samplePathological.setId(CustomIdGenerator.nextId());
        samplePathological.setSampleId(sampleId);
        //清空预实验相关的参数
        if (!pathological.getPreExperiment()) {
            samplePathological.setPreDate("")
                    .setPrePeople("")
                    .setPreSubjectName("")
                    .setPreApplicantId(null)
                    .setPreApplicantName("")
                    .setPreApplicantLeaderId(null)
                    .setPreApplicantLeaderName("")
                    .setPreContent("")
                    .setPreConclusion("");
        }
        samplePathological.insert();

        //保存预实验列表
        if (pathological.getPreExperiment()) {
            List<PathologicalPreExperiment> preExperiments = BeanUtil.copyToList(pathological.getExperiments(), PathologicalPreExperiment.class);
            preExperiments.forEach(item -> item.setPathologicalId(samplePathological.getId()));
            preExperimentService.saveBatch(preExperiments);
        }

        //保存附件列表
        List<FileInfo> fileInfos = new ArrayList<>();
        pathological.getSampleInfoFiles().forEach(f -> {
                    FileInfo fileInfo = new FileInfo()
                            .setRelateId(samplePathological.getId())
                            .setType(PATHOLOGICAL_SAMPLE_INFO)
                            .setName(f.getName())
                            .setUrl(f.getUrl());
                    fileInfos.add(fileInfo);
                });
        pathological.getAntibodyInfoFiles().forEach(f -> {
                    FileInfo fileInfo = new FileInfo()
                            .setRelateId(samplePathological.getId())
                            .setType(PATHOLOGICAL_ANTIBODY_INFO)
                            .setName(f.getName())
                            .setUrl(f.getUrl());
                    fileInfos.add(fileInfo);
                });
        if (CollUtil.isNotEmpty(fileInfos)) {
            fileInfoService.saveBatch(fileInfos);
        }

        log.info("保存病理样本送检信息成功");
        generate(sampleId);
    }

    @Override
    public void generate(Long sampleId) {
        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);
        SamplePathologicalVo pathologicalVo = getDetail(sampleId);

        //文件名称：课题名称-申请人-申请日期-随机四位.docx
        String applyDate = DateUtil.format(sampleInfo.getApplyTime(), DatePattern.CHINESE_DATE_PATTERN);
        String fileName = String.format("病理实验计划表-%s-%s-%s-%s.docx",
                StrUtil.nullToEmpty(pathologicalVo.getSubjectName()), StrUtil.nullToEmpty(sampleInfo.getApplicantName()),
                applyDate,
                RandomUtil.randomString(RandomUtil.BASE_CHAR_NUMBER_LOWER, 4));
        log.info("生成word 课题名称：{} 文件名称：{}", pathologicalVo.getSubjectName(), fileName);

        //切片数量为负数，重新设置为null
        Field[] intFields = ReflectUtil.getFields(SamplePathologicalVo.class, f -> f.getType().equals(Integer.class));
        for (Field field : intFields) {
            try {
                field.setAccessible(true);
                Integer value = (Integer) field.get(pathologicalVo);
                if (value!= null && value < 0) {
                    field.set(pathologicalVo, null);
                }
            } catch (IllegalAccessException e) {
                log.error("反射获取字段值失败", e);
            }
        }

        JSONObject dataMap = new JSONObject();
        wordDataService.fillDocCreator(dataMap);

        //富文本图片
        JSONArray images = new JSONArray();
        dataMap.put("images", images);

        //实验名称
        List<PreExperimentVo> experiments = pathologicalVo.getExperiments();
        AtomicInteger experimentIndex = new AtomicInteger(0);
        experiments.forEach(item -> {
            item.setTitle("实验" + Convert.numberToChinese(experimentIndex.incrementAndGet(), false));
            if (StrUtil.isNotBlank(item.getFblx())) {
                item.setFblx(item.getFblx().replace("未封闭", "未封"));
            }
            if (StrUtil.isNotBlank(item.getXftj())) {
                item.setXftj(item.getXftj().replace("高温高压", "温压"));
            }
            if (StrUtil.isNotBlank(item.getSyjg())) {
                HtmlParseVo parseVo = WordNodeFactory.html2xml(item.getSyjg());
                item.setSyjg(parseVo.getContent());
                images.addAll(parseVo.getImageCache());
            }
        });

        JSONObject sampleInfoObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(sampleInfo, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("sampleInfo", sampleInfoObj);
        JSONObject pathologicalObj = JSON.parseObject(JSON.toJSONStringWithDateFormat(pathologicalVo, DatePattern.NORM_DATETIME_PATTERN));
        dataMap.put("pathological", pathologicalObj);

        //申请时间
        String applyDateStr = DateUtil.format(sampleInfo.getApplyTime(), DatePattern.NORM_DATE_PATTERN);
        sampleInfoObj.put("applyDate", applyDateStr);

        //实验人员
        if (StrUtil.isNotEmpty(pathologicalVo.getPrePeople())) {
            String joinName = JSON.parseArray(pathologicalVo.getPrePeople())
                    .stream()
                    .map(item -> {
                        JSONObject obj = (JSONObject) item;
                        return obj.getString("nickname");
                    })
                    .collect(Collectors.joining("、"));
            pathologicalObj.put("prePeopleName", joinName);
        }

        String modelName = "animalPathological.ftl";
        fileInfoService.uploadFileWithFileName(modelName, fileName, sampleInfo.getId(), sampleInfo.getStatus(), dataMap);
    }

    @Override
    public SamplePathologicalVo getDetail(Long sampleId) {
        SamplePathological samplePathological = getOne(new LambdaQueryWrapper<>(SamplePathological.class)
                .eq(SamplePathological::getSampleId, sampleId)
                .eq(SamplePathological::getDeleted, WhetherEnum.NO.getCode()));
        if (samplePathological == null) {
            return null;
        }

        SamplePathologicalVo pathologicalVo = BeanUtil.copyProperties(samplePathological, SamplePathologicalVo.class);

        //预实验列表
        List<PathologicalPreExperiment> preExperiments = preExperimentService.list(new LambdaQueryWrapper<>(PathologicalPreExperiment.class)
                .eq(PathologicalPreExperiment::getPathologicalId, samplePathological.getId()));
        pathologicalVo.getExperiments().addAll(BeanUtil.copyToList(preExperiments, PreExperimentVo.class));

        //附件列表
        List<FileInfo> fileInfos = fileInfoService.list(new LambdaQueryWrapper<>(FileInfo.class)
                .eq(FileInfo::getRelateId, samplePathological.getId())
                .eq(FileInfo::getDeleted, WhetherEnum.NO.getCode())
                .orderByAsc(FileInfo::getId));
        if (CollUtil.isNotEmpty(fileInfos)) {
            pathologicalVo.getSampleInfoFiles().addAll(fileInfos.stream()
                    .filter(item -> PATHOLOGICAL_SAMPLE_INFO.equals(item.getType()))
                    .map(item -> BeanUtil.copyProperties(item, FileVo.class))
                    .collect(Collectors.toList()));

            pathologicalVo.getAntibodyInfoFiles().addAll(fileInfos.stream()
                    .filter(item -> PATHOLOGICAL_ANTIBODY_INFO.equals(item.getType()))
                    .map(item -> BeanUtil.copyProperties(item, FileVo.class))
                    .collect(Collectors.toList()));
        }

        return pathologicalVo;
    }
}